#ifdef HAVE_CONFIG_H
#include "config.h"
#endif
#ifdef JFB_ENABLE_SPLASH_SCREEN
#ifdef JFB_WITH_LIBPNG
#include <stdio.h>
#include <stdlib.h>
#include "read-png.h"
#include "message.h"

void
print_png_file_info(png_structp png_ptr, png_infop info_ptr)
{
	u_int width, height;
	int bit_depth, color, interlaced, compression, filter;
	png_get_IHDR(png_ptr, info_ptr, (png_uint_32 *)&width, (png_uint_32 *)&height,
		&bit_depth, &color, &interlaced, &compression, &filter);
	fprintf(stderr, "DEBUG - PNG image %dx%dx%d\n",
		width, height, bit_depth);
	fprintf(stderr, "DEBUG - Color type: %d, interlaced: %d, compression type: %d, filter method: %d\n", 
		color, interlaced, compression, filter);
}

u_char *
read_png_file(FILE *fp, int *width, int *height)
{
	char header[8] = {0, };
	int is_png = 0;
	int i, j;
	u_char *data;
	int color_type, bit_depth;

	png_structp png_ptr;
	png_infop info_ptr;
	png_infop end_info;
	png_bytep *row_pointers;

	fread(header, 1, READ_HEADER_BYTES, fp);
	is_png = !png_sig_cmp(header, 0, READ_HEADER_BYTES);
	if(!is_png) {
		print_message("(UXP Splash) read-png: ERROR: not a png file\n");
		return NULL;
	}

	png_ptr = png_create_read_struct(PNG_LIBPNG_VER_STRING, (png_voidp)NULL, NULL, NULL);
	if(!png_ptr) {
		print_message("(UXP Splash) read-png: ERROR: Creating png_ptr\n");
		return NULL;
	}
	info_ptr = png_create_info_struct(png_ptr);
	if(!info_ptr) {
		print_message("(UXP Splash) read-png: ERROR: Creating info_ptr\n");
		png_destroy_read_struct(&png_ptr, NULL, NULL);
		return NULL;
	}
	end_info = png_create_info_struct(png_ptr);
	if(!end_info) {
		print_message("(UXP Splash) read-png: ERROR: Creating end_ptr\n");
		png_destroy_read_struct(&png_ptr, &info_ptr, NULL);
		return NULL;
	}
	png_init_io(png_ptr, fp);
	png_set_sig_bytes(png_ptr, 7);

	png_read_info(png_ptr, info_ptr);

	*width = png_get_image_width(png_ptr, info_ptr);
	*height = png_get_image_height(png_ptr, info_ptr);
	bit_depth = png_get_bit_depth(png_ptr, info_ptr);
	color_type = png_get_color_type(png_ptr, info_ptr);

	if(color_type == PNG_COLOR_TYPE_PALETTE)
		png_set_palette_to_rgb(png_ptr);
	if(color_type == PNG_COLOR_TYPE_GRAY && bit_depth < 8)
		png_set_gray_1_2_4_to_8(png_ptr);
	if(png_get_valid(png_ptr, info_ptr, PNG_INFO_tRNS))
		png_set_tRNS_to_alpha(png_ptr);
	if(bit_depth == 16)
		png_set_strip_16(png_ptr);
	if(color_type && PNG_COLOR_MASK_ALPHA)
		png_set_strip_alpha(png_ptr);
	if(bit_depth < 8)
		png_set_packing(png_ptr);
	if(color_type == PNG_COLOR_TYPE_GRAY || color_type == PNG_COLOR_TYPE_GRAY_ALPHA)
		png_set_gray_to_rgb(png_ptr);

	if((row_pointers = (u_char **)malloc(*height * sizeof(u_char *))) == NULL) {
		png_destroy_read_struct(&png_ptr, &info_ptr, &end_info);
		return NULL;
	}
	for(i = 0; i < *height; i++) {
		if((row_pointers[i] = (u_char *)malloc(*width * 3 * sizeof(u_char))) == NULL) {
			png_destroy_read_struct(&png_ptr, &info_ptr, &end_info);
			for(j = 0; j < i; j++) {
				free(row_pointers[j]);
			}
			free(row_pointers);
			return NULL;
		}
	}	
	png_read_image(png_ptr, row_pointers);
	png_read_end(png_ptr, end_info);

	if((data = (u_char *)malloc(*width * *height * 3 * sizeof(u_char))) == NULL) {
		png_destroy_read_struct(&png_ptr, &info_ptr, &end_info);
		for(i = 0; i < *height; i++) {
			free(row_pointers[i]);
		}
		free(row_pointers);
		return NULL;
	}

	for(i = 0; i < *height; i++) {
		memcpy(data + i * (*width * 3), row_pointers[i], *width * 3);
	}

	png_destroy_read_struct(&png_ptr, &info_ptr, &end_info);
	for(i = 0; i < *height; i++) {
		free(row_pointers[i]);
	}
	free(row_pointers);
	return data;
}
#endif
#endif
